/* Part of Misnamed Filter Configurator (see Program.cs)
 * by Alex Vallat <www.byalexv.co.uk>
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Windows.Forms;

namespace MisnamedFilter
{
	public class Settings
	{
		private bool mAutoLearn;
		private List<AddressSet> mAddressSets;

		public Settings()
		{
			Reload();
		}

		public void Reload()
		{
			mAddressSets = new List<AddressSet>();
			mAutoLearn = true; //By default

			string filename = GetFileName();

			if (File.Exists(filename))
			{
				XmlDocument settings = new XmlDocument();
				try
				{
					settings.Load(filename);
				}
				catch (Exception e)
				{
					MessageBox.Show(String.Format("Could not load settings file from:\n\n  {0}\n\n{1}", filename, e.Message));
					settings = null;
				}

				if (settings != null)
				{
					mAutoLearn = XmlConvert.ToBoolean(settings.DocumentElement.GetAttribute("autoLearn"));
					foreach (XmlElement addressSetElement in settings.DocumentElement.SelectNodes("AddressSet"))
					{
						AddressSet addressSet = new AddressSet(addressSetElement);
						mAddressSets.Add(addressSet);
					}
				}
			}
		}

		public void Save()
		{
			XmlDocument settings = new XmlDocument();
			settings.AppendChild(settings.CreateElement("MisnamedFilter"));
			settings.DocumentElement.SetAttribute("autoLearn", XmlConvert.ToString(mAutoLearn));
			foreach (AddressSet addressSet in mAddressSets)
			{
				settings.DocumentElement.AppendChild(addressSet.Serialise(settings));
			}

			try
			{
				settings.Save(GetFileName());
			}
			catch (Exception e)
			{
				MessageBox.Show(String.Format("Could not save settings file to:\n\n  {0}\n\n{1}", GetFileName(), e.Message));
			}
		}

		private string GetFileName()
		{
			string baseSettingsPath;
			//Check to see whether using per-user or all-users settings

			if ((int)Microsoft.Win32.Registry.GetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Spamihilator", "ConfigDir", 1) == 0)
			{
				//0 = per-user
				baseSettingsPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), @"Spamihilator\plugins\Misnamed Filter");
			}
			else
			{
				//1 = all-users
				baseSettingsPath = Path.GetDirectoryName(typeof(Settings).Assembly.Location);
			}

#if FORCE_LOCAL_SETTIGS_FILE
			baseSettingsPath = Path.GetDirectoryName(typeof(Settings).Assembly.Location);
#endif

			return Path.Combine(baseSettingsPath, "Misnamed Filter.settings.xml");
		}

		public List<AddressSet> AddressSets { get { return mAddressSets; } }
		public bool AutoLearn
		{
			get { return mAutoLearn; }
			set { mAutoLearn = value; }
		}
	}
}
